/*
 Basic Door Bell example

  - connects to an MQTT server
  - publishes "hello world" to the topic "outTopic"
  - subscribes to the topic "inTopic"
  - Use the serial moniter to see arrived messages w/ Baud rate 115200
  - If DOORBELL_PIN goes HIGH, send a signal to take a picture
  
*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

/************************* WiFi Access Point *********************************/

#define WLAN_SSID       "SDNet"
#define WLAN_PASS       "CapstoneProject"

/************************* Adafruit.io Setup *********************************/

#define SERVER_ADDRESS      "10.176.58.5"  // server in 213 SR

#define SERVER_PORT         1883        // standard port
#define DOORBELL_PIN        4


void callback(const MQTT::Publish& pub) {
  // handle message arrived
  Serial.print("Message arrived [");
  Serial.print(pub.topic());
  Serial.print("] ");

  Serial.println(pub.payload_string());

  Serial.println();
}// end of callback function


// Create an ESP8266 WiFiClient class to connect to the MQTT server.
WiFiClient wf_client; // instantiate wifi client
PubSubClient client(wf_client, SERVER_ADDRESS); // pass to pubsub


void setup() {
  // Setup console
  Serial.begin(115200);
  delay(10);
  Serial.println();
  Serial.println();
  Serial.println(F("Modified pubsub client basic code using modified pubsub software"));
  client.set_callback(callback);

  // Connect to WiFi access point.
  Serial.println(); Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);

  WiFi.begin(WLAN_SSID, WLAN_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();

  Serial.println("WiFi connected");
  Serial.println("IP address: "); Serial.println(WiFi.localIP());

  //Set pin as Input
  pinMode(DOORBELL_PIN, INPUT);
  delay(10);

}

void loop() {


  if (WiFi.status() == WL_CONNECTED) {
    if (!client.connected()) {
      if (client.connect("mydevice")) {
        Serial.println("Connected to MQTT server");
        client.publish("outTopic", "hello world");
        Serial.println("Just published hello world to outTopic");
        client.subscribe("inTopic");
        Serial.println("Just subscribed to inTopic");
      }
    }
    /* wait for incoming messages */

    if (client.connected())
      client.loop();

      // If doorbell is pressed, send a "take picture" signal
      int val=digitalRead(DOORBELL_PIN);
      if (val==HIGH){
        client.publish("inTopic", "Yes");
        delay(100);
      }
  }
}

