/*
 Basic Solenoid Scratch

  - connects to an MQTT server
  - publishes "AWUPWUPWUPWUP" to the topic "Zoidburg"
  - subscribes to the topic SOL_SUB_TOPIC defined below
  - Use the serial moniter to see arrived messages w/ Baud rate 38400
  - If a float is sent between 0 and MAX_OPEN_LEN seconds, the door will open for that long
    
*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>


/************************* WiFi Access Point *********************************/

#define WLAN_SSID       "SDNet"
#define WLAN_PASS       "CapstoneProject"

/************************* Adafruit.io Setup *********************************/

#define SERVER_ADDRESS      "10.176.58.5"  // server in 213 SR

#define SERVER_PORT         1883        // standard port

/************************* Camera-DevThing Parameters ************************/

#define SOL_SUB_TOPIC "Open_Door"     // Topic Solenoid is listening to
#define OPEN_DOOR_ACK "yes"           // ACK to Open Door
#define SOL_OUT_PIN 5                // Output Pin to Control Solenoid
#define MAX_OPEN_LEN 60               // Max amount of time (in seconds) the solenoid can be on
String inString="";                   // Used for input string

/************************* Functions *****************************************/

// Create an ESP8266 WiFiClient class to connect to the MQTT server.
WiFiClient wf_client; // instantiate wifi client
PubSubClient client(wf_client, SERVER_ADDRESS); // pass to pubsub


void callback(const MQTT::Publish& pub);
void OpenDoor(int t);

/************************* Setup Console ******** ****************************/

void setup() {
  // Setup console
  Serial.begin(38400);
  delay(500);
  
  Serial.println();
  Serial.println();
  Serial.println(F("Modified pubsub client basic code using modified pubsub software"));
  client.set_callback(callback);

  // Connect to WiFi access point.
  Serial.println(); Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);

  WiFi.begin(WLAN_SSID, WLAN_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();

  Serial.println("WiFi connected");
  Serial.println("IP address: "); Serial.println(WiFi.localIP());
  delay(50);

  pinMode(SOL_OUT_PIN, OUTPUT);
  digitalWrite(SOL_OUT_PIN, HIGH);

}

void loop() {

  if (WiFi.status() == WL_CONNECTED) {
    if (!client.connected()) {
      if (client.connect("mydevice")) {
        Serial.println("Connected to MQTT server");
        client.publish("Zoidburg", "AWUPWUPWUPWUPWUP");
        client.subscribe(SOL_SUB_TOPIC);
      }
    }
  }

    /* wait for incoming messages */

    if (client.connected())
      client.loop();
}

/************************* Functions *****************************************/

void callback(const MQTT::Publish& pub) {
  // handle message arrived
  Serial.print("Message arrived [");
  Serial.print(pub.topic());
  Serial.print("] ");

  inString=pub.payload_string();
  Serial.println(inString);
  delay(10);

  // Convert message to float
  float inFloat=0;
  inFloat=inString.toFloat();
  OpenDoor(inFloat);
  
}// end of callback function

// Opens door for t seconds
void OpenDoor(int t){
  digitalWrite(SOL_OUT_PIN, LOW);
  int i=0;
  while (i<t*1000){
    i=i+100;
    delay(100);
  }
  digitalWrite(SOL_OUT_PIN, HIGH);
}

