/*
 Setup ESP as Server Scratch

  - Sets up as an MQTT server
  - Waits for Network and Server Parameters to be sent to server
  - Subscribes to topic SETUP_TOPIC
  - Once it's connected to the correct server, it will become an access point
  
*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#define ESP8266_LED 5
#define SETUP_TOPIC "Setup"



/************************* WiFi Access Point *********************************/

String Wlan_Ssid="Test";//               "SDNet";
String Wlan_Pass="Test";//               "CapstoneProject";

/************************* Adafruit.io Setup *********************************/

bool connected = false;
String Server_Address="Test";//"10.176.58.5";           // server in 213 SR "10.176.58.5"

int SERVER_PORT=1883;              // standard port
int WLAN_STEP=0;
#define ESP_SERVER_ADDRESS "192.168.4.1"

const char WiFiAPPSK[] = "LockStars"; // Password when in server mode


void callback(const MQTT::Publish& pub);
void setupWiFiAP();
void stopWiFiAP();


// Create an ESP8266 WiFiClient class to connect to the MQTT server.
WiFiClient wf_client; // instantiate wifi client
PubSubClient client(wf_client, ESP_SERVER_ADDRESS); // pass to pubsub
//WiFiServer server(80);


void setup() {
  // Setup console
  Serial.begin(38400);
  delay(2000);
  Serial.println();
  Serial.println();
  Serial.println(F("Modified pubsub client basic code using modified pubsub software"));
  
  // Convert Strings to char
  char WLAN_SSID[100];
  char WLAN_PASS[100];
  char SERVER_ADDRESS[100];
  Wlan_Ssid.toCharArray(WLAN_SSID, sizeof(WLAN_SSID));
  Wlan_Pass.toCharArray(WLAN_PASS, sizeof(WLAN_PASS));
  Server_Address.toCharArray(SERVER_ADDRESS, sizeof(SERVER_ADDRESS));
  Serial.print("WLAN_SSID: ");
  Serial.println(WLAN_SSID);
  Serial.print("WLAN_PASS: ");
  Serial.println(WLAN_PASS);
  Serial.print("SERVER_ADDRESS: ");
  Serial.print(SERVER_ADDRESS);

  // Create PubSubClient Instance
  client = PubSubClient(wf_client, SERVER_ADDRESS);
  
  // Connect to WiFi server
  Serial.println(); Serial.println();
  Serial.print("Connecting to ");
  Serial.println(Wlan_Ssid);
  delay(100);

  int attemptNum=0;
  int EndFlag=0;
  int i=0;
  WiFi.begin(WLAN_SSID, WLAN_PASS);
  while ((WiFi.status() != WL_CONNECTED)&&(EndFlag==0)) {
    delay(500);
    Serial.print(".");
    attemptNum++;
    if (attemptNum>16){
      attemptNum=0;
      Serial.println("");
      Serial.println("Could Not Connect");
      EndFlag=1;  
    }    
  }

  // Set client callback if connected
  if (WiFi.status() == WL_CONNECTED) {
    client.set_callback(callback);
    Serial.println();
    Serial.println("WiFi connected");
    Serial.println("IP address: "); Serial.println(WiFi.localIP());
  }

}

void loop() {

  // If WiFi is connected, run program
  if (WiFi.status() == WL_CONNECTED) {
    if (!client.connected()) {
      if (client.connect("mydevice")) {
        Serial.println("Connected to MQTT server");
        client.publish("outTopic", "hello world");
        Serial.println("Just published hello world to outTopic");
        client.subscribe("inTopic");
        Serial.println("Just subscribed to inTopic");
      }
    }
    /* wait for incoming messages */

    if (client.connected())
      client.loop();
  }
  // If WiFi is not connected, start as access point and look for WiFi information
  else {
    setupWiFiAP();
    WiFiServer server(80);
    server.begin();
    delay(100);
    WiFiClient esp_client= server.available();
    if (!esp_client){
      return;
    }
    String trash;
    String inString;
    while(!esp_client.available()){
    }
    // Read information sent to access point
    // WLAN_SSID
    trash=esp_client.readStringUntil('/');
    Wlan_Ssid=esp_client.readStringUntil('/');

    // WLAN_PASS
    Wlan_Pass=esp_client.readStringUntil('/');

    // SERVER_ADDRESS
    Server_Address=esp_client.readStringUntil(' ');
    trash=esp_client.readStringUntil('\r');
    esp_client.flush();
    
    stopWiFiAP();
    setup();
  }
  
}


void callback(const MQTT::Publish& pub) {
  // handle message arrived
  Serial.print("Message arrived [");
  Serial.print(pub.topic());
  Serial.print("] ");

  Serial.println(pub.payload_string());
  
  Serial.println();
}// end of callback function

// Create a WiFi access point
void setupWiFiAP(){
  
  WiFi.mode(WIFI_AP);

  // Do a little work to get a unique-ish name. Append the
  // last two bytes of the MAC (HEX'd) to "Thing-":
  uint8_t mac[WL_MAC_ADDR_LENGTH];
  WiFi.softAPmacAddress(mac);
  String macID = String(mac[WL_MAC_ADDR_LENGTH - 2], HEX) +
                 String(mac[WL_MAC_ADDR_LENGTH - 1], HEX);
  macID.toUpperCase();
  String AP_NameString = "ESP8266 Thing " + macID;

  char AP_NameChar[AP_NameString.length() + 1];
  memset(AP_NameChar, 0, AP_NameString.length() + 1);

  for (int i=0; i<AP_NameString.length(); i++)
    AP_NameChar[i] = AP_NameString.charAt(i);

  WiFi.softAP(AP_NameChar, WiFiAPPSK);
}

// Kill WiFi access point and start as a server
void stopWiFiAP(){

  WiFi.mode(WIFI_OFF);
  WiFi.softAPdisconnect(true);
  WiFi.mode(WIFI_STA);
  
}

